import unittest
import numpy as np
import numpy.testing as npt
from wilson import wcxf
import wilson
from wilson.run.smeft.smpar import p
import ckmutil
from math import pi, log
from wilson.util.wetutil import C_symm_keys
import wilson.match.smeft_loop
import wilson.match.smeft_tree


np.random.seed(39)

# generate a random WC instance for the SMEFT Warsaw basis
C_Warsaw_random = {}
basis = wcxf.Basis['SMEFT', 'Warsaw']
for sector, wcs in basis.sectors.items():
    for name, d in wcs.items():
         C_Warsaw_random[name] = 1e-6*np.random.rand()
         if 'real' not in d or d['real'] == False:
             C_Warsaw_random[name] += 1j*1e-6*np.random.rand()


class TestMatch(unittest.TestCase):
    def test_match_qq3_1122(self):
        # tests matching of Q_qq^(3) of Warsaw onto O_ud^V8,LL in JMS basis
        from_wc =  wcxf.WC(values = {'qq3_1122': 2e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        V = ckmutil.ckm.ckm_tree(p["Vus"], p["Vub"], p["Vcb"], p["delta"])
        self.assertAlmostEqual(to_wc['V8udLL_1221']/V[0,0].conjugate()
        /V[1,1].conjugate(),8e-6)

    def test_match_qq3_1322(self):
        # tests matching of Q_qq^(3) of Warsaw onto O_ud^V8,LL in JMS basis
        from_wc =  wcxf.WC(values = {'qq3_1322': 3e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        V = ckmutil.ckm.ckm_tree(p["Vus"], p["Vub"], p["Vcb"], p["delta"])
        self.assertAlmostEqual(to_wc['V8udLL_1223']/V[2,2].conjugate()
        /V[1,1].conjugate(),12e-6)

    def test_match_ll_1212(self):
        # tests matching of Q_ll of Warsaw onto O_nue^V,LL in JMS basis
        from_wc =  wcxf.WC(values = {'ll_1212': 2e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        self.assertAlmostEqual(to_wc['VnueLL_1212'],4e-6)

    def test_match_ll_1312(self):
        # tests matching of Q_ll of Warsaw onto O_nue^V,LL in JMS basis
        from_wc =  wcxf.WC(values = {'ll_1213': 20e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        self.assertAlmostEqual(to_wc['VnueLL_1312'],20e-6)

    def test_match_lq1_1233(self):
        # tests matching of Q_lq^1 of Warsaw onto O_ed^V,LL in JMS basis
        from_wc =  wcxf.WC(values = {'lq1_1233': 12e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        self.assertAlmostEqual(to_wc['VedLL_1233'],12e-6)

    def test_match_ee_1233(self):
        # tests matching of Q_ee of Warsaw onto O_ee^V,RR in JMS basis
        from_wc =  wcxf.WC(values = {'ee_1233': 100e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        self.assertAlmostEqual(to_wc['VeeRR_1233'],100e-6)

    def test_match_uu_1112(self):
        # tests matching of Q_uu of Warsaw onto O_uu^V,RR in JMS basis
        from_wc =  wcxf.WC(values = {'uu_1112': 5e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        self.assertAlmostEqual(to_wc['VuuRR_1112'],5e-6)

    def test_match_dd_1223(self):
        # tests matching of Q_dd of Warsaw onto O_dd^V,RR in JMS basis
        from_wc =  wcxf.WC(values = {'dd_1223': 51e-6} ,
                    scale = 1e3 , eft = 'SMEFT' , basis = 'Warsaw up')
        to_wc = from_wc.match('WET', 'JMS')
        self.assertAlmostEqual(to_wc['VddRR_1223'],51e-6)


class TestRun(unittest.TestCase):
    def test_run_lq3_3333(self):
        w = wilson.Wilson({'lq3_2333': 1e-6}, 1000, 'SMEFT', 'Warsaw')
        # determine g at input scale
        g = wilson.run.smeft.SMEFT(w.wc).C_in['g']
        # run down
        wc = w.match_run(100, 'SMEFT', 'Warsaw')
        # compare LL to expected value
        sf = 2  # symmetry factor since our 2333 is 2* larger
        self.assertAlmostEqual(wc['ll_2333'],
        sf * 1e-6 / (16 * pi**2) * (-g**2) * log(100 / 1000))


class TestMatchingSymmetryFactors(unittest.TestCase):
    def test_match_symmfac(self):
        """Test that the WET WCs coming out of the matching fulfill
        the correct symmetry relations (namely, have the same symmetries
        as the operators)."""
        C_SMEFT = wilson.util.smeftutil.wcxf2arrays_symmetrized(C_Warsaw_random)
        C = wilson.match.smeft_tree.match_all_array(C_SMEFT, p)
        for k in C:
            if k in C_symm_keys[41] + C_symm_keys[4] + C_symm_keys[6]:
                a = np.einsum('klij', C[k]) # C_ijkl = C_klij
                npt.assert_array_almost_equal(C[k], a, err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[5] + C_symm_keys[4] + C_symm_keys[6]:
                a = np.einsum('jilk', C[k]).conj() # C_ijkl = C_jilk*
                npt.assert_array_almost_equal(C[k], a, err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[4] + C_symm_keys[6]:
                a = np.einsum('lkji', C[k]).conj() # C_ijkl = C_lkji*
                npt.assert_array_almost_equal(C[k], a, err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[6]:
                a = np.einsum('ilkj', C[k]) # C_ijkl = C_ilkj
                npt.assert_array_almost_equal(C[k], a, err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[9]:
                a = -np.einsum('jikl', C[k]) # C_ijkl = -C_jikl
                npt.assert_array_almost_equal(C[k], a, err_msg="Failed for {}".format(k),
                                              decimal=20)

    def test_match_symmfac_loop(self):
        """Test that the WET WCs coming out of the matching fulfill
        the correct symmetry relations (namely, have the same symmetries
        as the operators)."""
        C_SMEFT = wilson.util.smeftutil.wcxf2arrays_symmetrized(C_Warsaw_random)
        C = wilson.match.smeft_loop.match_all_array(C_SMEFT, p, scale=120)
        for k in C:
            if k in C_symm_keys[41] + C_symm_keys[4] + C_symm_keys[6]:
                a = np.einsum('klij', C[k]) # C_ijkl = C_klij
                npt.assert_array_almost_equal(np.array(C[k], complex), np.array(a, complex), err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[5] + C_symm_keys[4] + C_symm_keys[6]:
                a = np.einsum('jilk', C[k]).conj() # C_ijkl = C_jilk*
                npt.assert_array_almost_equal(np.array(C[k], complex), np.array(a, complex), err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[4] + C_symm_keys[6]:
                a = np.einsum('lkji', C[k]).conj() # C_ijkl = C_lkji*
                npt.assert_array_almost_equal(np.array(C[k], complex), np.array(a, complex), err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[6]:
                a = np.einsum('ilkj', C[k]) # C_ijkl = C_ilkj
                npt.assert_array_almost_equal(np.array(C[k], complex), np.array(a, complex), err_msg="Failed for {}".format(k),
                                              decimal=20)
            if k in C_symm_keys[9]:
                a = -np.einsum('jikl', C[k]) # C_ijkl = -C_jikl
                npt.assert_array_almost_equal(np.array(C[k], complex), np.array(a, complex), err_msg="Failed for {}".format(k),
                                              decimal=20)



import unittest
from math import pi, sqrt

import numpy as np

from wilson import wcxf
import wilson
from wilson.parameters import p
from wilson.test_wilson import get_random_wc


np.random.seed(56)


class TestHiggs(unittest.TestCase):
    def test_higgs_warsaw_up(self):
        # test cgg, a trivial case
        for t in ["", "tilde"]:
            for up in ["", " up"]:
                wc = wcxf.WC("SMEFT", "Higgs-Warsaw up", 120, {"cgg" + t: 1e-8})
                wcW = wc.translate("Warsaw" + up)
                gs = sqrt(4 * pi * p["alpha_s"])
                self.assertEqual(wcW.dict, {"phiG" + t: 1e-8 * gs ** 2 / 4})

    def test_roundtrip_warsaw(self):
        wc = get_random_wc("SMEFT", "Warsaw up", 120)
        basis = wcxf.Basis["SMEFT", "Warsaw up"]
        for name in basis.all_wcs:
            if name == 'phiD':
                continue
            wc = wcxf.WC("SMEFT", "Warsaw up", 120, {name: 1e-8})
            wc_translated = wc.translate("Higgs-Warsaw up")
            wc_translated.validate()
            # translate back and check that nothing changed
            wc_roundtrip = wc_translated.translate("Warsaw up")
            for k, v in wc.dict.items():
                self.assertAlmostEqual(
                    v, wc_roundtrip.dict[k], places=20, msg="Failed for {}".format(k)
                )



"""Matcher from the SMEFT 'Warsaw up' basis to the WET JMS basis.

Based on arXiv:1908.05295."""


import numpy as np
from math import sqrt, pi
from wilson import wcxf
import wilson
from wilson.run.smeft.smpar import p as default_parameters
from wilson.util import smeftutil, wetutil
from wilson.match import smeft_tree, smeft_loop


def match_all(d_SMEFT, scale, parameters=None):
    """Match the SMEFT Warsaw basis onto the WET JMS basis.
    
    The optional `parameters` dictionary allows to overwrite the default
    numerical input parameters (such as CKM elements and quark masses).
    Moreover, there is a key `'loop_order'` which, if set to 1, allows
    to switch on the one-loop matching contributions (which are)
    omitted by default.
    """
    p = default_parameters.copy()
    if parameters is not None:
        # if parameters are passed in, overwrite the default values
        p.update(parameters)
    C = wilson.util.smeftutil.wcxf2arrays_symmetrized(d_SMEFT)
    C_WET_tree = smeft_tree.match_all_array(C, p)
    if p.get('loop_order') == 1:
        # One loop matching only added if 'loop_order' is 1!
        C_WET_loop = smeft_loop.match_all_array(C, p, scale=scale)
        C_WET = {k: np.array(C_WET_tree[k] + C_WET_loop[k], complex) for k in C_WET_tree}
    else:
        C_WET = C_WET_tree
    C_WET = wilson.translate.wet.rotate_down(C_WET, p)
    C_WET = wetutil.unscale_dict_wet(C_WET)
    d_WET = wilson.util.smeftutil.arrays2wcxf(C_WET)
    basis = wcxf.Basis['WET', 'JMS']
    keys = set(d_WET.keys()) & set(basis.all_wcs)
    d_WET = {k: d_WET[k] for k in keys}
    return d_WET



"""Matcher from the SMEFT 'Warsaw up' basis to the WET JMS basis.

Based on arXiv:1709.04486."""


import numpy as np
from math import sqrt, pi
from wilson import wcxf
import wilson
from wilson.parameters import p as default_parameters
from wilson.util import smeftutil, wetutil

# Based on arXiv:1709.04486

# CONSTANTS

Nc = 3

def _match_all_array(C, p):
    
    # AUXILIARY FUNCTIONS

    # Eq. (6.4)
    GF = p['GF']
    GFx = GF - sqrt(2)/4 * ( -C['ll'][1,0,0,1] - C['ll'][0,1,1,0] + 2*C['phil3'][1,1] + 2*C['phil3'][0,0] )

    vT = sqrt(1/sqrt(2)/abs(GFx))
    eps = C["phiWB"] * vT**2
    g2b = 2*p["m_W"]/vT

    alpha_e = p['alpha_e']
    eb = sqrt(4*pi*alpha_e)
    g1b = eb*g2b/sqrt(g2b**2-eb**2) + eb**2*g2b/(g2b**2-eb**2) * eps
    sb = g1b/sqrt(g1b**2+g2b**2) * (1 + eps/2. * g2b/g1b * ((g2b**2-g1b**2)/(g1b**2+g2b**2)))
    cb = g2b/sqrt(g1b**2+g2b**2) * (1 - eps/2. * g1b/g2b * ((g2b**2-g1b**2)/(g1b**2+g2b**2)))

    p['m_Z'] = sqrt(vT**2 / 4 * (1 + vT**2 / 2 * C["phiD"]) * (g2b**2 + g1b**2) + vT**2 / 2 * eps * g1b * g2b)

    # Eq. (2.26)
    eb = g2b*sb - 1/2.*cb*g2b* vT**2*C["phiWB"]
    gzb = eb/(sb*cb) * (1 + (g1b**2+g2b**2)/(2*g1b*g2b)*vT**2*C["phiWB"])

    # Eq. (2.30)
    wl = np.eye(3)+vT**2*C["phil3"]
    wq = np.eye(3)+vT**2*C["phiq3"]
    wr = 1/2.*vT**2*C["phiud"]
    znu = np.eye(3)*1/2.-1/2.*vT**2*C["phil1"]+1/2.*vT**2*C["phil3"]
    zel = np.eye(3)*(-1/2.+sb**2)-1/2.*vT**2*C["phil1"]-1/2.*vT**2*C["phil3"]
    zer = np.eye(3)*sb**2-1/2.*vT**2*C["phie"]
    zul = np.eye(3)*(1/2.-2./3*sb**2)-1/2.*vT**2*C["phiq1"]+1/2.*vT**2*C["phiq3"]
    zur = np.eye(3)*(-2./3)*sb**2-1/2.*vT**2*C["phiu"]
    zdl = np.eye(3)*(-1/2.+1/3.*sb**2)-1/2.*vT**2*C["phiq1"]-1/2.*vT**2*C["phiq3"]
    zdr = np.eye(3)*(1/3.)*sb**2-1/2.*vT**2*C["phid"]

    # MATCHING CONDITIONS
    c = {}

    # Table 9
    # c["nu"] = 1/2.*C["llphiphi"]* vT**2

    # Table 10
    # c["nugamma"] = np.zeros((3,3))

    # Table 11
    c["egamma"] = 1/sqrt(2) * (-C["eW"] * sb + C["eB"] * cb) * vT

    c["ugamma"] = 1/sqrt(2) * (C["uW"] * sb + C["uB"] * cb) * vT
    c["dgamma"] = 1/sqrt(2) * (-C["dW"] * sb + C["dB"] * cb) * vT
    c["uG"] = 1/sqrt(2) * C["uG"] * vT
    c["dG"] = 1/sqrt(2) * C["dG"] * vT

    #Table 12
    c["G"] = C["G"]
    c["Gtilde"] = C["Gtilde"]

    # Table 13
    c["VnunuLL"] = C["ll"]-gzb**2/(4*p["m_Z"]**2)*np.einsum('pr,st',znu,znu)-gzb**2/(4*p["m_Z"]**2)*np.einsum('pt,sr',znu,znu)
    c["VeeLL"] = C["ll"]-gzb**2/(4*p["m_Z"]**2)*np.einsum('pr,st',zel,zel)-gzb**2/(4*p["m_Z"]**2)*np.einsum('pt,sr',zel,zel)

    c["VnunuLL"] = (c["VnunuLL"] + np.einsum('ilkj', c["VnunuLL"])) / 2
    c["VeeLL"] = (c["VeeLL"] + np.einsum('ilkj', c["VeeLL"])) / 2

    c["VnueLL"] = C["ll"]+np.einsum('stpr',C["ll"])-g2b**2/(2*p["m_W"]**2)*np.einsum('pt,rs',wl,wl.conjugate())-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',znu,zel)

    c["VnuuLL"] = C["lq1"]+C["lq3"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',znu,zul)
    c["VnudLL"] = C["lq1"]-C["lq3"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',znu,zdl)
    c["VeuLL"] = C["lq1"]-C["lq3"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zel,zul)
    c["VedLL"] = C["lq1"]+C["lq3"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zel,zdl)
    c["VnueduLL"] = 2*C["lq3"]-g2b**2/(2*p["m_W"]**2)*np.einsum('pr,ts',wl,wq.conjugate())
    # + h.c.

    c["VuuLL"] = C["qq1"]+C["qq3"]-gzb**2/(2*p["m_Z"]**2)*np.einsum('pr,st',zul,zul)
    c["VddLL"] = C["qq1"]+C["qq3"]-gzb**2/(2*p["m_Z"]**2)*np.einsum('pr,st',zdl,zdl)
    c["V1udLL"] = C["qq1"]+np.einsum('stpr',C["qq1"])-C["qq3"]-np.einsum('stpr',C["qq3"])+2/Nc*np.einsum('ptsr',C["qq3"])+2/Nc*np.einsum('srpt',C["qq3"])-g2b**2/(2*p["m_W"]**2)*np.einsum('pt,rs',wq,wq.conjugate())/Nc-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zul,zdl)
    c["V8udLL"] = 4*np.einsum('ptsr',C["qq3"])+4*np.einsum('srpt',C["qq3"])-g2b**2/(p["m_W"]**2)*np.einsum('pt,rs',wq,wq.conjugate())


    # Table 14
    c["VeeRR"] = C["ee"]-gzb**2/(4*p["m_Z"]**2)*np.einsum('pr,st',zer,zer)-gzb**2/(4*p["m_Z"]**2)*np.einsum('pt,sr',zer,zer)

    c["VeuRR"] = C["eu"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zer,zur)
    c["VedRR"] = C["ed"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zer,zdr)

    c["VuuRR"] = C["uu"]-gzb**2/(2*p["m_Z"]**2)*np.einsum('pr,st',zur,zur)
    c["VddRR"] = C["dd"]-gzb**2/(2*p["m_Z"]**2)*np.einsum('pr,st',zdr,zdr)
    c["V1udRR"] = C["ud1"]-g2b**2/(2*p["m_W"]**2)*np.einsum('pt,rs',wr,wr.conjugate())/Nc-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zur,zdr)
    c["V8udRR"] = C["ud8"]-g2b**2/(p["m_W"]**2)*np.einsum('pt,rs',wr,wr.conjugate())


    # Table 15
    c["VnueLR"] = C["le"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',znu,zer)
    c["VeeLR"] = C["le"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zel,zer)

    c["VnuuLR"] = C["lu"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',znu,zur)
    c["VnudLR"] = C["ld"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',znu,zdr)
    c["VeuLR"] = C["lu"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zel,zur)
    c["VedLR"] = C["ld"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zel,zdr)
    c["VueLR"] = C["qe"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zul,zer)
    c["VdeLR"] = C["qe"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zdl,zer)
    c["VnueduLR"] = -g2b**2/(2*p["m_W"]**2)*np.einsum('pr,ts',wl,wr.conjugate())
    #+ h.c.

    c["V1uuLR"] = C["qu1"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zul,zur)
    c["V8uuLR"] = C["qu8"]
    c["V1udLR"] = C["qd1"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zul,zdr)
    c["V8udLR"] = C["qd8"]
    c["V1duLR"] = C["qu1"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zdl,zur)
    c["V8duLR"] = C["qu8"]
    c["V1ddLR"] = C["qd1"]-gzb**2/(p["m_Z"]**2)*np.einsum('pr,st',zdl,zdr)
    c["V8ddLR"] = C["qd8"]
    c["V1udduLR"] = -g2b**2/(2*p["m_W"]**2)*np.einsum('pr,ts',wq,wr.conjugate())
    c["V8udduLR"] = np.zeros((3,3,3,3))


    # Table 16
    c["SeuRL"] = np.zeros((3,3,3,3))
    c["SedRL"] = C["ledq"]
    c["SnueduRL"] = C["ledq"]


    # Table 17
    c["SeeRR"] = np.zeros((3,3,3,3))

    c["SeuRR"] = -C["lequ1"]
    c["TeuRR"] = -C["lequ3"]
    c["SedRR"] = np.zeros((3,3,3,3))
    c["TedRR"] = np.zeros((3,3,3,3))
    c["SnueduRR"] = C["lequ1"]
    c["TnueduRR"] = C["lequ3"]

    c["S1uuRR"] = np.zeros((3,3,3,3))
    c["S8uuRR"] = np.zeros((3,3,3,3))
    c["S1udRR"] = C["quqd1"]
    c["S8udRR"] = C["quqd8"]
    c["S1ddRR"] = np.zeros((3,3,3,3))
    c["S8ddRR"] = np.zeros((3,3,3,3))
    c["S1udduRR"] = -np.einsum('stpr',C["quqd1"])
    c["S8udduRR"] = -np.einsum('stpr',C["quqd8"])

    # # Table 18
    # c["SnunuLL"] = np.zeros((3,3,3,3))

    # # Table 19
    # c["SnueLL"] = np.zeros((3,3,3,3))
    # c["TnueLL"] = np.zeros((3,3,3,3))
    # c["SnueLR"] = np.zeros((3,3,3,3))

    # c["SnuuLL"] = np.zeros((3,3,3,3))
    # c["TnuuLL"] = np.zeros((3,3,3,3))
    # c["SnuuLR"] = np.zeros((3,3,3,3))
    # c["SnudLL"] = np.zeros((3,3,3,3))
    # c["TnudLL"] = np.zeros((3,3,3,3))
    # c["SnudLR"] = np.zeros((3,3,3,3))
    # c["SnueduLL"] = np.zeros((3,3,3,3))
    # c["TnueduLL"] = np.zeros((3,3,3,3))
    # c["SnueduLR"] = np.zeros((3,3,3,3))
    # c["VnueduRL"] = np.zeros((3,3,3,3))
    # c["VnueduRR"] = np.zeros((3,3,3,3))

    # Table 20
    c["SuddLL"] = np.einsum('srpt',C["qqql"])-np.einsum('rspt',C["qqql"])+np.einsum('rpst',C["qqql"])
    c["SduuLL"] = c["SuddLL"]
    c["SuudLR"] = np.zeros((3,3,3,3))
    c["SduuLR"] = -C["qque"]-np.einsum('rpst',C["qque"])
    c["SuudRL"] = np.zeros((3,3,3,3))
    c["SduuRL"] = C["duql"]
    c["SdudRL"] = -C["duql"]
    c["SdduRL"] = np.zeros((3,3,3,3))
    c["SduuRR"] = C["duue"]

    # # Table 21
    # c["SdddLL"] = np.zeros((3,3,3,3))
    # c["SuddLR"] = np.zeros((3,3,3,3))
    # c["SdduLR"] = np.zeros((3,3,3,3))
    # c["SdddLR"] = np.zeros((3,3,3,3))
    # c["SdddRL"] = np.zeros((3,3,3,3))
    # c["SuddRR"] = np.zeros((3,3,3,3))
    # c["SdddRR"] = np.zeros((3,3,3,3))
    return c


def match_all_array(C_SMEFT, p):
    # generate a dictionary with 0 Wilson coefficients = Standard Model
    C_SMEFT_0 = {k: 0*v for k, v in C_SMEFT.items()}
    # compute the SMEFT matching contribution but subtract the SM part
    match_C = _match_all_array(C_SMEFT, p)
    match_C0 = _match_all_array(C_SMEFT_0, p)
    return {k: match_C[k] - match_C0[k] for k in match_C}




"""Extraction of Standard Model parameters taking into account dimension-six contributions."""


from math import sqrt, pi
import numpy as np
import ckmutil
import ckmutil.ckm
import scipy.optimize
from cmath import phase
import warnings


# Default values for SM parameters: MSbar parameters at M_Z (except GF)
p = {}
p['GF'] = 1.1663787e-5
p['alpha_e'] = 1 / 127.9
p['alpha_s'] = 0.1185
p['Vus'] = 0.2243
p['Vub'] = 3.62e-3
p['Vcb'] = 4.221e-2
p['delta'] = 1.27
p['m_e'] = 0.000511
p['m_mu'] = 0.1057
p['m_tau'] = 1.777
p['m_u'] = 0.00127  # mu(2 GeV)=0.0022
p['m_c'] = 0.635  # mc(mc)=1.28
p['m_d'] = 0.00270  # md(2 GeV)=0.0047
p['m_s'] = 0.0551  # ms(2 GeV)=0.095
p['m_b'] = 2.85  # mb(mb)=4.18
# MSbar running masses at MZ computed with "mr"
# https://github.com/apik/mr, https://arxiv.org/abs/1601.08143
p['m_t'] = 169.0
p['m_W'] = 80.20
p['m_Z'] = 91.46
p['m_h'] = 130.6


def m2Lambda_to_vMh2(m2, Lambda, C):
    """Function to numerically determine the  physical Higgs VEV and mass
    given the parameters of the Higgs potential.

    The relations used by this function have been derived from the Lagrangian
    in eq. (3.1) of arXiv:1704.03888. This paper presents results expanded up to
    linear order in the Wilson coefficients while the all-order expressions are
    implemented here.
    """
    Cphi = C['phi'].real
    Ckin = C['phiBox'].real - C['phiD'].real / 4
    if abs(Cphi) < 1e-16:
        v2 = 2 * m2 / Lambda
    else:
        sqrt_arg = Lambda**2 - 12 * Cphi * m2
        if not sqrt_arg >= 0:
            raise ValueError("'Lambda**2 - 12 * Cphi * m2' must be positive.")
        v2 = ( Lambda - sqrt(sqrt_arg) )/( 3 * Cphi )
    if not v2 > 0:
        raise ValueError('No solution with positive v2.')
    Mh2 = v2 / ( 1 - 2 * Ckin * v2 ) * ( Lambda - 3 * Cphi * v2 )
    if not Mh2 > 0:
        raise ValueError('No solution with positive Mh2.')
    return {'v': sqrt(v2), 'Mh2': Mh2}

def vMh2_to_m2Lambda(v, Mh2, C):
    """Function to numerically determine the parameters of the Higgs potential
    given the physical Higgs VEV and mass.

    The relations used by this function have been derived from the Lagrangian
    in eq. (3.1) of arXiv:1704.03888. This paper presents results expanded up to
    linear order in the Wilson coefficients while the all-order expressions are
    implemented here."""
    if not v > 0 or not Mh2 > 0:
        raise ValueError('`v` and `Mh2` are expected to be positive.')
    v2 = v**2
    Cphi = C['phi'].real
    Ckin = C['phiBox'].real - C['phiD'].real / 4
    Lambda = Mh2 / v2 - 2 * Ckin * Mh2 + 3 * Cphi * v2
    m2 = Lambda * v2 / 2 - 3/4 * Cphi * v2**2
    return {'m2': m2, 'Lambda': Lambda}

def get_gpbar(ebar, gbar, v, C):
    r"""Function to numerically determine the hypercharge gauge coupling
    in terms of $\bar e$, $\bar g$, v, and the Wilson coefficients."""
    if C['phiWB'] == 0:  # this is the trivial case
        gpbar = ebar * gbar / sqrt(gbar**2 - ebar**2)
    else:  # if epsilon != 0, need to iterate
        def f0(x):  # we want the root of this function
            gpb = x
            gb = gbar
            eps = C['phiWB'].real * (v**2)
            ebar_calc = (gb * gpb / sqrt(gb**2 + gpb**2) *
                        (1 - eps * gb * gpb / (gb**2 + gpb**2)))
            return (ebar_calc - ebar).real
        try:
            gpbar = scipy.optimize.brentq(f0, 0, 3)
        except (scipy.optimize.nonlin.NoConvergence, ValueError) as e:
            raise ValueError("No solution for gp found. This problem can be caused by very large values for one or several Wilson coefficients.")
    return gpbar * (1 - C['phiB'].real * (v**2))


def smeftpar(scale, C, basis):
    """Get the running parameters in SMEFT."""
    # start with a zero dict and update it with the input values
    MW = p['m_W']
    # MZ = p['m_Z']
    GF = p['GF']
    Mh = p['m_h']
    vb = sqrt(1 / sqrt(2) / GF)
    v = vb  # TODO
    _d = vMh2_to_m2Lambda(v=v, Mh2=Mh**2, C=C)
    m2 = _d['m2'].real
    Lambda = _d['Lambda'].real
    gsbar = sqrt(4 * pi * p['alpha_s'])
    gs = (1 - C['phiG'].real * (v**2)) * gsbar
    gbar = 2 * MW / v
    g = gbar * (1 - C['phiW'].real * (v**2))
    ebar = sqrt(4 * pi * p['alpha_e'])
    gp = get_gpbar(ebar, gbar, v, C)
    c = {}
    c['m2'] = m2
    c['Lambda'] = Lambda
    c['g'] = g
    c['gp'] = gp
    c['gs'] = gs
    K = ckmutil.ckm.ckm_tree(p['Vus'], p['Vub'], p['Vcb'], p['delta'])
    if basis == 'Warsaw':
        Mu = K.conj().T @ np.diag([p['m_u'], p['m_c'], p['m_t']])
        Md = np.diag([p['m_d'], p['m_s'], p['m_b']])
    elif basis == 'Warsaw up':
        Mu = np.diag([p['m_u'], p['m_c'], p['m_t']])
        Md = K @ np.diag([p['m_d'], p['m_s'], p['m_b']])
    else:
        raise ValueError("Basis '{}' not supported".format(basis))
    Me = np.diag([p['m_e'], p['m_mu'], p['m_tau']])
    c['Gd'] = Md / (v / sqrt(2)) + C['dphi'] * (v**2) / 2
    c['Gu'] = Mu / (v / sqrt(2)) + C['uphi'] * (v**2) / 2
    c['Ge'] = Me / (v / sqrt(2)) + C['ephi'] * (v**2) / 2
    return c


def smpar(C):
    """Get the running effective SM parameters."""
    m2 = C['m2'].real
    Lambda = C['Lambda'].real
    vMh2 = m2Lambda_to_vMh2(m2, Lambda, C)
    v = vMh2['v']
    Mh2 = vMh2['Mh2']
    GF = 1 / (sqrt(2) * v**2)  # TODO
    eps = C['phiWB'].real * (v**2)
    gb = (C['g'] / (1 - C['phiW'].real * (v**2))).real
    gpb = (C['gp'] / (1 - C['phiB'].real * (v**2))).real
    gsb = (C['gs'] / (1 - C['phiG'].real * (v**2))).real
    MW = gb * v / 2
    ZG0 = 1 + C['phiD'].real * (v**2) / 4
    MZ = (sqrt(gb**2 + gpb**2) / 2 * v
          * (1 + eps * gb * gpb / (gb**2 + gpb**2)) * ZG0)
    Mnup = -(v**2) * C['llphiphi']
    Mep = v / sqrt(2) * (C['Ge'] - C['ephi'] * (v**2) / 2)
    Mup = v / sqrt(2) * (C['Gu'] - C['uphi'] * (v**2) / 2)
    Mdp = v / sqrt(2) * (C['Gd'] - C['dphi'] * (v**2) / 2)
    UeL, Me, UeR = ckmutil.diag.msvd(Mep)
    UuL, Mu, UuR = ckmutil.diag.msvd(Mup)
    UdL, Md, UdR = ckmutil.diag.msvd(Mdp)
    UnuL, Mnu = ckmutil.diag.mtakfac(Mnup)
    eb = (gb * gpb / sqrt(gb**2 + gpb**2) *
          (1 - eps * gb * gpb / (gb**2 + gpb**2)))
    K = UuL.conj().T @ UdL
    # U = UeL.conj().T @ UnuL
    sm = {}
    sm['GF'] = GF
    sm['alpha_e'] = eb**2 / (4 * pi)
    sm['alpha_s'] = gsb**2 / (4 * pi)
    sm['Vub'] = abs(K[0, 2])
    sm['Vcb'] = abs(K[1, 2])
    sm['Vus'] = abs(K[0, 1])
    sm['delta'] = phase(-K[0, 0] * K[0, 2].conj()
                        / (K[1, 0] * K[1, 2].conj()))
    # sm['U'] = Uu
    sm['m_W'] = MW
    sm['m_Z'] = MZ
    sm['m_h'] = sqrt(abs(Mh2))
    sm['m_u'] = Mu[0]
    sm['m_c'] = Mu[1]
    sm['m_t'] = Mu[2]
    sm['m_d'] = Md[0]
    sm['m_s'] = Md[1]
    sm['m_b'] = Md[2]
    sm['m_e'] = Me[0]
    sm['m_mu'] = Me[1]
    sm['m_tau'] = Me[2]
    return  {k: v.real for k, v in sm.items()}



"""Compare the new tree-level SMEFT matching based on aXiv:1908.05295
to the old one based on arXiv:1709.04486."""

import unittest
import numpy as np
import numpy.testing as npt
from wilson import wcxf
import wilson
from wilson.test_wilson import get_random_wc
import wilson.match._smeft_old
import wilson.match.smeft_tree
from wilson.parameters import p


np.random.seed(77)


# These WCs appear quadratically in the "old" matching implementation
wc_sm = ['phiD', 'phiWB']
wc_vert = [k for k in wcxf.Basis['SMEFT', 'Warsaw'].all_wcs
          if 'phiq' in k
          or 'phil' in k
           or 'phiu' in k
           or 'phid' in k
           or 'phie' in k
          ]


_wcr = get_random_wc('SMEFT', 'Warsaw', 100, cmax=1e-8)
wc_linear = wilson.Wilson({k: v for k, v in _wcr.dict.items()
                     if k not in wc_sm + wc_vert}, 100, 'SMEFT', 'Warsaw').wc
wc_quadratic = wilson.Wilson({k: v for k, v in _wcr.dict.items()
                     if k in wc_sm + wc_vert}, 100, 'SMEFT', 'Warsaw').wc


class TestSMEFTWETreimpl(unittest.TestCase):
    def test_linear(self):
        """For WCs entering linearly, agreement should be numerically exact"""
        C = wilson.util.smeftutil.wcxf2arrays_symmetrized(wc_linear.dict)
        c_old = wilson.match._smeft_old.match_all_array(C, p)
        c_new = wilson.match.smeft_tree.match_all_array(C, p)
        for k in c_old:
            npt.assert_almost_equal(c_old[k], c_new[k], decimal=18,
                                    err_msg="Failed for {}".format(k))

    def test_quadratic(self):
        """For WCs entering also quadratically, agreement should be good
        up to the size of quadratic terms"""
        C = wilson.util.smeftutil.wcxf2arrays_symmetrized(wc_quadratic.dict)
        c_old = wilson.match._smeft_old.match_all_array(C, p)
        c_new = wilson.match.smeft_tree.match_all_array(C, p)
        for k in c_old:
            npt.assert_almost_equal(c_old[k], c_new[k], decimal=10,
                                    err_msg="Failed for {}".format(k))



"""Functions for translating to and from the SMEFT Higgs-like basis.

The code was adapted from the Rosetta package:
https://github.com/kenmimasu/Rosetta/tree/master/Rosetta
(c) Adam Falkowski, Benjamin Fuks, Kentarou Mawatari, 
Ken Mimasu, Veronica Sanz & Francesco Riva
"""

from math import sqrt, pi

import ckmutil
import numpy as np

from wilson import wcxf
from wilson.parameters import p as default_parameters


def higgslike_to_warsaw_up(C, parameters=None, sectors=None):
    """Translate from the Higgs-Warsaw basis to the Warsaw up
    basis."""

    basis = wcxf.Basis["SMEFT", "Higgs-Warsaw up"]
    H = {k: C.get(k, 0) for k in basis.all_wcs}

    p = default_parameters.copy()
    if parameters is not None:
        # if parameters are passed in, overwrite the default values
        p.update(parameters)
    V = ckmutil.ckm.ckm_tree(p["Vus"], p["Vub"], p["Vcb"], p["delta"])
    Vdag = V.T.conjugate()

    W = H.copy()

    # parameters
    # neglecting dim.-6 corrections here as these are higher-order
    # when multiplying dim.-6 WCs!
    v = 1 / sqrt(sqrt(2) * p["GF"])  # Higgs VEV
    lam = p["m_h"] ** 2 / (2 * v ** 2)  # Higgs self-coupling
    e = sqrt(4 * pi * p["alpha_e"])
    s2w = 1 / 2 * (1 - sqrt(1 - (4 * pi * p['alpha_e']) / (sqrt(2) * p['GF'] * p['m_Z']**2)))
    gs = sqrt(4 * pi * p["alpha_s"])
    sw = sqrt(s2w)
    g = gw = e / sw
    c2w = 1 - s2w
    cw = sqrt(c2w)
    gp = e / cw
    gw2 = g ** 2
    gp2 = gp ** 2
    gs2 = gs ** 2

    dM = (
        H["deltagWlL_11"] + H["deltagWlL_22"] - H["ll_1221"] / 2
    ) / 2  # W mass correction

    W["phi"] = (
        3 * lam * H["deltacz"]
        + 8 * lam * dM
        - H["deltalambda3"]
        + (
            H["czBox"] * gw2
            + H["czz"] * gp2
            - H["czgamma"] * gp2 * (c2w - s2w)
            - H["cgammagamma"] * gw2 * s2w ** 2
        )
        * 4
        * gw2
        * lam
        / (gw2 - gp2)
    )

    W["phiBox"] = (
        H["deltacz"]
        + 2 * dM
        + (
            H["czBox"] * (3 * gw2 - gp2)
            + 2 * H["czz"] * gp2
            - 2 * H["czgamma"] * gp2 * (c2w - s2w)
            - 2 * H["cgammagamma"] * gw2 * s2w ** 2
        )
        * gw2
        / (gw2 - gp2)
        / 2
    )

    W["phiD"] = (
        -(
            H["czBox"]
            + H["czz"]
            - H["czgamma"] * (c2w - s2w)
            - H["cgammagamma"] * s2w * c2w
        )
        * 2
        * gw2
        * gp2
        / (gw2 - gp2)
        - 4 * dM
    )

    W["phiG"] = H["cgg"] * gs2 / 4

    W["phiW"] = (
        (H["czz"] + H["czgamma"] * 2 * s2w + H["cgammagamma"] * s2w ** 2) * gw2 / 4
    )

    W["phiB"] = (
        (H["czz"] - H["czgamma"] * 2 * c2w + H["cgammagamma"] * c2w ** 2) * gp2 / 4
    )

    W["phiWB"] = (
        (H["czz"] - H["czgamma"] * (c2w - s2w) - H["cgammagamma"] * c2w * s2w)
        * gw
        * gp
        / 2
    )

    W["phiGtilde"] = H["cggtilde"] * gs2 / 4

    W["phiWtilde"] = (
        (H["czztilde"] + H["czgammatilde"] * 2 * s2w + H["cgammagammatilde"] * s2w ** 2)
        * gw2
        / 4
    )

    W["phiBtilde"] = (
        (H["czztilde"] - H["czgammatilde"] * 2 * c2w + H["cgammagammatilde"] * c2w ** 2)
        * gp2
        / 4
    )

    W["phiWtildeB"] = (
        (
            H["czztilde"]
            - H["czgammatilde"] * (c2w - s2w)
            - H["cgammagammatilde"] * c2w * s2w
        )
        * gw
        * gp
        / 2
    )

    C1 = (
        (
            H["czBox"] * gw2
            + H["czz"] * gp2
            - H["czgamma"] * gp2 * (c2w - s2w)
            - H["cgammagamma"] * gw2 * s2w ** 2
        )
        * gw2
        / (gw2 - gp2)
    )

    C2 = (
        (
            H["czBox"]
            + H["czz"]
            - H["czgamma"] * (c2w - s2w)
            - H["cgammagamma"] * s2w * c2w
        )
        * gw2
        * gp2
        / (gw2 - gp2)
    )

    # rotate deltagZdL to up basis
    deltagZdL = np.zeros((3, 3), complex)
    for i in range(3):
        for j in range(3):
            if j >= i:
                ind = "_{}{}".format(i + 1, j + 1)
                deltagZdL[i, j] = H["deltagZdL" + ind]
    # symmetrize
    deltagZdL = deltagZdL + deltagZdL.T.conjugate() - np.diag(np.diag(deltagZdL))
    deltagZdLrot = V @ deltagZdL @ Vdag

    facp = np.eye(3) * C1
    fac = np.eye(3) * C2

    for i in range(3):
        for j in range(3):
            ind = "_{}{}".format(i + 1, j + 1)
            if j >= i:
                W["phil3" + ind] = H["deltagWlL" + ind] + facp[i, j] / 2
                W["phiq3" + ind] = (
                    H["deltagZuL" + ind] - deltagZdLrot[i, j] + facp[i, j] / 2
                )
                W["phil1" + ind] = (
                    -2 * H["deltagZeL" + ind] - H["deltagWlL" + ind] + fac[i, j] / 2
                )
                W["phiq1" + ind] = (
                    -H["deltagZuL" + ind] - deltagZdLrot[i, j] - fac[i, j] / 6
                )
                W["phie" + ind] = -2 * H["deltagZeR" + ind] + fac[i, j]
                W["phiu" + ind] = -2 * H["deltagZuR" + ind] - 2 / 3 * fac[i, j]
                W["phid" + ind] = -2 * H["deltagZdR" + ind] + 1 / 3 * fac[i, j]

            W["phiud" + ind] = -2 * H["deltagWqR" + ind]

    warsaw = wcxf.Basis["SMEFT", "Warsaw up"]
    all_wcs = set(warsaw.all_wcs)  # to speed up lookup
    return {k: v for k, v in W.items() if k in all_wcs}


def warsaw_up_to_higgslike(C, parameters=None, sectors=None):

    basis = wcxf.Basis["SMEFT", "Warsaw up"]
    W = {k: C.get(k, 0) for k in basis.all_wcs}
    H = W.copy()

    p = default_parameters.copy()
    if parameters is not None:
        # if parameters are passed in, overwrite the default values
        p.update(parameters)
    V = ckmutil.ckm.ckm_tree(p["Vus"], p["Vub"], p["Vcb"], p["delta"])
    Vdag = V.T.conjugate()

    # parameters
    # neglecting dim.-6 corrections here as these are higher-order
    # when multiplying dim.-6 WCs!
    v = 1 / sqrt(sqrt(2) * p["GF"])  # Higgs VEV
    lam = p["m_h"] ** 2 / (2 * v ** 2)  # Higgs self-coupling
    e = sqrt(4 * pi * p["alpha_e"])
    s2w = 1 / 2 * (1 - sqrt(1 - (4 * pi * p['alpha_e']) / (sqrt(2) * p['GF'] * p['m_Z']**2)))
    gs = sqrt(4 * pi * p["alpha_s"])
    sw = sqrt(s2w)
    g = gw = e / sw
    c2w = 1 - s2w
    cw = sqrt(c2w)
    gp = e / cw
    gw2 = g ** 2
    gp2 = gp ** 2
    gs2 = gs ** 2

    def f(T3, Q, i, j):  # [eqn (4.11)]
        if i == j:
            Acoeff = -gw * gp / (gw2 - gp2) * W["phiWB"]
            Zcoeff = (
                W["ll_1221"] / 4
                - W["phil3_11"].real / 2
                - W["phil3_22"].real / 2
                - W["phiD"] / 4
            )
            return Acoeff * Q + Zcoeff * (T3 + Q * gp2 / (gw2 - gp2))
        else:
            return 0

    # rotate Cphiq3 + Cphiq1 to down basis
    Cphiq1 = np.zeros((3, 3), complex)
    Cphiq3 = np.zeros((3, 3), complex)
    for i in range(3):
        for j in range(3):
            if j >= i:
                ind = "_{}{}".format(i + 1, j + 1)
                Cphiq1[i, j] = W["phiq1" + ind]
                Cphiq3[i, j] = W["phiq3" + ind]
    # symmetrize
    Cphiq1 = Cphiq1 + Cphiq1.T.conjugate() - np.diag(np.diag(Cphiq1))
    Cphiq3 = Cphiq3 + Cphiq3.T.conjugate() - np.diag(np.diag(Cphiq3))
    Cphiq13rot = Vdag @ (Cphiq3 + Cphiq1) @ V

    # W/Z chiral coupling deviations
    for i in range(3):
        for j in range(3):
            ind = "_{}{}".format(i + 1, j + 1)
            if j >= i:
                H["deltagWlL" + ind] = (
                    W["phil3" + ind] + f(1 / 2, 0, i, j) - f(-1 / 2, -1, i, j)
                )

                H["deltagZeL" + ind] = (
                    -1 / 2 * W["phil3" + ind]
                    - 1 / 2 * W["phil1" + ind]
                    + f(-1 / 2, -1, i, j)
                )

                H["deltagZeR" + ind] = -1 / 2 * W["phie" + ind] + f(0, -1, i, j)

                H["deltagZuL" + ind] = (
                    1 / 2 * W["phiq3" + ind]
                    - 1 / 2 * W["phiq1" + ind]
                    + f(1 / 2, 2 / 3, i, j)
                )

                H["deltagZdL" + ind] = -1 / 2 * Cphiq13rot[i, j] + f(
                    -1 / 2, -1 / 3, i, j
                )

                H["deltagZuR" + ind] = -1 / 2 * W["phiu" + ind] + f(0, 2 / 3, i, j)

                H["deltagZdR" + ind] = -1 / 2 * W["phid" + ind] + f(0, -1 / 3, i, j)

            H["deltagWqR" + ind] = -W["phiud" + ind] / 2

    H["deltalambda3"] = (
        lam
        * (
            3 * W["phiBox"]
            - 3 / 4 * W["phiD"]
            + 1 / 4 * W["ll_1221"]
            - W["phil3_11"] / 2
            - W["phil3_22"] / 2
        )
        - W["phi"]
    )

    H["deltacz"] = (
        W["phiBox"]
        - W["phiD"] / 4
        + 3 / 4 * W["ll_1221"]
        - 3 / 2 * W["phil3_11"]
        - 3 / 2 * W["phil3_22"]
    )

    # Two derivative field strength interactions
    H["czBox"] = (
        -W["ll_1221"] / 2 + W["phiD"] / 2 + W["phil3_11"] + W["phil3_22"]
    ) / gw2

    H["cgg"] = (4 / gs2) * W["phiG"]

    H["cgammagamma"] = 4 * (W["phiW"] / gw2 + W["phiB"] / gp2 - W["phiWB"] / gw / gp)

    H["czz"] = (
        4
        * (gw2 * W["phiW"] + gp2 * W["phiB"] + gw * gp * W["phiWB"])
        / (gw2 + gp2) ** 2
    )

    H["czgamma"] = (
        4 * W["phiW"] - 4 * W["phiB"] - 2 * (gw2 - gp2) / (gw * gp) * W["phiWB"]
    ) / (gw2 + gp2)

    H["cggtilde"] = (4 / gs2) * W["phiGtilde"]

    H["cgammagammatilde"] = 4 * (
        W["phiWtilde"] / gw2 + W["phiBtilde"] / gp2 - W["phiWtildeB"] / gw / gp
    )

    H["czztilde"] = (
        4
        * (gw2 * W["phiWtilde"] + gp2 * W["phiBtilde"] + gw * gp * W["phiWtildeB"])
        / (gw2 + gp2) ** 2
    )

    H["czgammatilde"] = (
        4 * W["phiWtilde"]
        - 4 * W["phiBtilde"]
        - 2 * (gw2 - gp2) / (gw * gp) * W["phiWtildeB"]
    ) / (gw2 + gp2)

    basis = wcxf.Basis["SMEFT", "Higgs-Warsaw up"]
    all_wcs = set(basis.all_wcs)  # to speed up lookup
    return {k: v for k, v in H.items() if k in all_wcs}



from setuptools import find_packages, setup

with open("wilson/_version.py") as f:
    exec(f.read())

with open("README.md", encoding="utf-8") as f:
    LONG_DESCRIPTION = f.read()

setup(
    name="wilson",
    version=__version__,
    author="Jason Aebischer, David M. Straub",
    author_email="jason.aebischer@tum.de, straub@protonmail.com",
    url="https://github.com/wilson-eft/wilson",
    description="Toolkit for effective field theories beyond the Standard Model",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    license="MIT",
    packages=find_packages(),
    package_data={
        "wilson": [
            "run/smeft/tests/data/*.*",
            "data/tests/*.*",
            "data/*.*",
            "wcxf/data/*.yml",
            "wcxf/data/*.yaml",
            "wcxf/data/*.json",
            "wcxf/bases/*.json",
            "wcxf/bases/child/*.json",
        ]
    },
    install_requires=[
        "numpy>=1.16.5",
        "scipy>=1.0",
        "pandas",
        "pylha>=0.2",
        "pyyaml",
        "ckmutil>=0.3.2",
        "rundec>=0.5",
        "voluptuous",
    ],
    extras_require={"testing": ["nose"]},
    entry_points={
        "console_scripts": [
            "wcxf = wilson.wcxf.cli:wcxf_cli",
            "wcxf2eos = wilson.wcxf.cli:eos",
            "wcxf2dsixtools = wilson.wcxf.cli:wcxf2dsixtools",
            "dsixtools2wcxf = wilson.wcxf.cli:dsixtools2wcxf",
            "wcxf2smeftsim = wilson.wcxf.cli:smeftsim",
        ]
    },
)



import unittest
import os
from tempfile import mkdtemp
import subprocess
import pylha
from shutil import rmtree
from wilson import wcxf


my_path = os.path.dirname(os.path.realpath(__file__))
data_path = os.path.join(my_path, '..', 'data')


class TestSMEFTsim(unittest.TestCase):
    def test_wcxf2smeftsim(self):
        for testfile in ['test.Warsaw_mass.yml', 'test.Warsaw_mass_incomplete.yml']:
            tmpdir = mkdtemp()
            # use default settings
            res = subprocess.run(['wcxf2smeftsim',
                                  os.path.join(data_path, testfile)],
                                 cwd=tmpdir)
            # check return code
            self.assertEqual(res.returncode, 0, msg="Command failed")
            # check if file is present
            outf = os.path.join(tmpdir, 'wcxf2smeftsim_param_card.dat')
            self.assertTrue(os.path.isfile, outf)
            # check if can be imported as LHA
            with open(outf, 'r') as f:
                card = pylha.load(f)
            # check dict is not empty
            self.assertTrue(card)
            # remove tmpdir
            rmtree(tmpdir)

    def test_symm_fac(self):
        wc = wcxf.WC('SMEFT', 'Warsaw mass', 120,
                     {'ll_1111': 1e-6, 'll_1221': 4e-6})
        tmpdir = mkdtemp()
        with open(os.path.join(tmpdir, 'my_wcxf.yaml'), 'w') as f:
            wc.dump(f, fmt='yaml')
        res = subprocess.run(['wcxf2smeftsim', 'my_wcxf.yaml'],
                             cwd=tmpdir)
        # check if file is present
        outf = os.path.join(tmpdir, 'wcxf2smeftsim_param_card.dat')
        self.assertTrue(os.path.isfile, outf)
        with open(outf, 'r') as f:
            card = pylha.load(f)
        self.assertEqual(dict(card['BLOCK']['FRBlock']['values'])[691],
                         1e6 * 4e-6 / 2,  # symmetry factor of 1 / 2!
                         msg="Wrong value for ll_1221")
        self.assertEqual(dict(card['BLOCK']['FRBlock']['values'])[689],
                         1e6 * 1e-6,  # no symmetry factor!
                         msg="Wrong value for ll_1111")
        # remove tmpdir
        # rmtree(tmpdir)
